@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -36,13 +36,14 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: rsh_locl.h,v 1.14 1999/02/11 21:02:39 joda Exp $ */
+/* $Id: rsh_locl.h,v 1.21 1999/08/04 17:04:12 assar Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <stdio.h>
+#include <assert.h>
 #include <stdarg.h>
 #include <ctype.h>
 #ifdef HAVE_SYS_TYPES_H
@@ -76,6 +77,9 @@
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
+#ifdef HAVE_SHADOW_H
+#include <shadow.h>
+#endif
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
@@ -99,6 +103,7 @@
 #include <prot.h>
 #endif
 #include <krb5.h>
+#include <kafs.h>
 
 #ifndef _PATH_NOLOGIN
 #define _PATH_NOLOGIN   "/etc/nologin"
@@ -131,11 +136,9 @@ extern des_cblock iv;
 #define USERNAME_SZ 16
 #define COMMAND_SZ 1024
 
-#define RSH_BUFSIZ 10240
+#define RSH_BUFSIZ (16 * 1024)
 
 #define PATH_RSH BINDIR "/rsh"
 
 ssize_t do_read (int fd, void *buf, size_t sz);
 ssize_t do_write (int fd, void *buf, size_t sz);
-ssize_t net_read (int fd, void *buf, size_t len);
-ssize_t net_write (int fd, const void *buf, size_t len);
