@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -37,7 +37,7 @@
  */
 
 #include "rsh_locl.h"
-RCSID("$Id: rshd.c,v 1.17 1999/02/11 21:02:40 joda Exp $");
+RCSID("$Id: rshd.c,v 1.27 1999/10/17 10:26:45 assar Exp $");
 
 enum auth_method auth_method;
 
@@ -47,8 +47,24 @@ krb5_crypto crypto;
 des_key_schedule schedule;
 des_cblock iv;
 
+krb5_ccache ccache, ccache2;
+int kerberos_status = 0;
+
 int do_encrypt = 0;
 
+static int do_unique_tkfile           = 0;
+static char tkfile[MAXPATHLEN] = "";
+
+static int do_inetd = 1;
+static char *port_str;
+static int do_rhosts;
+static int do_kerberos = 0;
+static int do_vacuous = 0;
+static int do_log = 1;
+static int do_newpag = 1;
+static int do_version;
+static int do_help = 0;
+
 static void
 syslog_and_die (const char *m, ...)
 {
@@ -92,8 +108,8 @@ read_str (int s, char *str, size_t sz, char *expl)
 
 static int
 recv_bsd_auth (int s, u_char *buf,
-	       struct sockaddr_in thisaddr,
-	       struct sockaddr_in thataddr,
+	       struct sockaddr_in *thisaddr,
+	       struct sockaddr_in *thataddr,
 	       char *client_username,
 	       char *server_username,
 	       char *cmd)
@@ -106,7 +122,7 @@ recv_bsd_auth (int s, u_char *buf,
     pwd = getpwnam(server_username);
     if (pwd == NULL)
 	fatal(s, "Login incorrect.");
-    if (iruserok(thataddr.sin_addr.s_addr, pwd->pw_uid == 0,
+    if (iruserok(thataddr->sin_addr.s_addr, pwd->pw_uid == 0,
 		 client_username, server_username))
 	fatal(s, "Login incorrect.");
     return 0;
@@ -115,8 +131,8 @@ recv_bsd_auth (int s, u_char *buf,
 #ifdef KRB4
 static int
 recv_krb4_auth (int s, u_char *buf,
-		struct sockaddr_in thisaddr,
-		struct sockaddr_in thataddr,
+		struct sockaddr *thisaddr,
+		struct sockaddr *thataddr,
 		char *client_username,
 		char *server_username,
 		char *cmd)
@@ -145,8 +161,8 @@ recv_krb4_auth (int s, u_char *buf,
 			   &ticket,
 			   "rcmd",
 			   instance,
-			   &thataddr,
-			   &thisaddr,
+			   (struct sockaddr_in *)thataddr,
+			   (struct sockaddr_in *)thisaddr,
 			   &auth,
 			   "",
 			   schedule,
@@ -164,60 +180,76 @@ recv_krb4_auth (int s, u_char *buf,
     syslog(LOG_INFO|LOG_AUTH,
 	   "kerberos v4 shell from %s on %s as %s, cmd '%.80s'",
 	   krb_unparse_name_long(auth.pname, auth.pinst, auth.prealm),
-	   inet_ntoa(thataddr.sin_addr),
+
+	   inet_ntoa(((struct sockaddr_in *)thataddr)->sin_addr),
 	   server_username,
 	   cmd);
 
+    memcpy (iv, auth.session, sizeof(iv));
+
     return 0;
 }
 
 #endif /* KRB4 */
 
+static int 
+save_krb5_creds (int s,
+                 krb5_auth_context auth_context,
+                 krb5_principal client)
+
+{
+    int ret;
+    krb5_data remote_cred;
+ 
+    krb5_data_zero (&remote_cred);
+    ret= krb5_read_message (context, (void *)&s, &remote_cred);
+    if (ret) {
+	krb5_data_free(&remote_cred);
+	return 0;
+    }
+    if (remote_cred.length == 0)
+	return 0;
+ 
+    ret = krb5_cc_gen_new(context, &krb5_mcc_ops, &ccache);
+    if (ret) {
+	krb5_data_free(&remote_cred);
+	return 0;
+    }
+  
+    krb5_cc_initialize(context,ccache,client);
+    ret = krb5_rd_cred(context, auth_context, ccache,&remote_cred);
+    krb5_data_free (&remote_cred);
+    if (ret)
+	return 0;
+    return 1;
+}
+
 static void
-recv_krb5_creds (int s,
-		 krb5_auth_context auth_context,
-		 char *username,
-		 krb5_principal client)
+krb5_start_session (void)
 {
     krb5_error_code ret;
-    krb5_ccache ccache;
-    krb5_data data;
-    char ccname[MAXPATHLEN];
-    struct passwd *pwd;
 
-    krb5_data_zero (&data);
-    ret = krb5_read_message (context,
-			     (void *)&s,
-			     &data);
-    if (ret || data.length == 0)
+    ret = krb5_cc_resolve (context, tkfile, &ccache2);
+    if (ret) {
+	krb5_cc_destroy(context, ccache);
 	return;
+    }
 
-    pwd = getpwnam (username);
-    if (pwd == NULL)
-	goto out;
+    ret = krb5_cc_copy_cache (context, ccache, ccache2);
+    if (ret) {
+	krb5_cc_destroy(context, ccache);
+	return ;
+    }
 
-    snprintf (ccname, sizeof(ccname),
-	      "FILE:/tmp/krb5cc_%u", pwd->pw_uid);
-    ret = krb5_cc_resolve (context, ccname, &ccache);
-    if (ret)
-	goto out;
-    ret = krb5_cc_initialize (context, ccache, client);
-    if (ret)
-	goto out;
-    ret = krb5_rd_cred (context, auth_context, ccache, &data);
-    krb5_cc_close (context, ccache);
-    if (ret)
-	goto out;
-    chown (ccname + 5, pwd->pw_uid, -1);
-
-out:
-    krb5_data_free (&data);
+    krb5_cc_close(context, ccache2);
+    krb5_cc_destroy(context, ccache);
+    return;
 }
 
 static int
 recv_krb5_auth (int s, u_char *buf,
-		struct sockaddr_in thisaddr,
-		struct sockaddr_in thataddr,
+		struct sockaddr *thisaddr,
+		struct sockaddr *thataddr,
 		char *client_username,
 		char *server_username,
 		char *cmd)
@@ -239,8 +271,6 @@ recv_krb5_auth (int s, u_char *buf,
 	|| memcmp (buf, KRB5_SENDAUTH_VERSION, len) != 0)
 	syslog_and_die ("bad sendauth version: %.8s", buf);
     
-    krb5_init_context (&context);
-
     status = krb5_sock_to_principal (context,
 				     s,
 				     "host",
@@ -265,7 +295,7 @@ recv_krb5_auth (int s, u_char *buf,
 
     read_str (s, server_username, USERNAME_SZ, "remote username");
     read_str (s, cmd, COMMAND_SZ, "command");
-    read_str (s, client_username, USERNAME_SZ, "local username");
+    read_str (s, client_username, COMMAND_SZ, "local username");
 
     status = krb5_auth_con_getkey (context, auth_context, &keyblock);
     if (status)
@@ -280,7 +310,7 @@ recv_krb5_auth (int s, u_char *buf,
     
     cksum_data.length = asprintf ((char **)&cksum_data.data,
 				  "%u:%s%s",
-				  ntohs(thisaddr.sin_port),
+				  ntohs(socket_get_port (thisaddr)),
 				  cmd,
 				  server_username);
 
@@ -295,7 +325,29 @@ recv_krb5_auth (int s, u_char *buf,
 
     free (cksum_data.data);
 
-    recv_krb5_creds (s, auth_context, server_username, ticket->client);
+    if (strncmp (client_username, "-u ", 3) == 0) {
+	do_unique_tkfile = 1;
+	memmove (client_username, client_username + 3,
+		 strlen(client_username) - 2);
+    }
+
+    if (strncmp (client_username, "-U ", 3) == 0) {
+	char *end, *temp_tkfile;
+
+	do_unique_tkfile = 1;
+	if (strncmp (server_username + 3, "FILE:", 5) == 0) {
+	    temp_tkfile = tkfile;
+	} else {
+	    strcpy (tkfile, "FILE:");
+	    temp_tkfile = tkfile + 5;
+	}
+	end = strchr(client_username + 3,' ');
+	strncpy(temp_tkfile, client_username + 3, end - client_username - 3);
+	temp_tkfile[end - client_username - 3] = '\0';
+	memmove (client_username, end +1, strlen(end+1)+1);
+    }
+
+    kerberos_status = save_krb5_creds (s, auth_context, ticket->client);
 
     if(!krb5_kuserok (context,
 		     ticket->client,
@@ -313,10 +365,18 @@ recv_krb5_auth (int s, u_char *buf,
 	char *name;
 
 	if (krb5_unparse_name (context, ticket->client, &name) == 0) {
+	    char addr_str[256];
+
+	    if (inet_ntop (thataddr->sa_family,
+			   socket_get_address (thataddr),
+			   addr_str, sizeof(addr_str)) == NULL)
+		strlcpy (addr_str, "unknown address",
+				 sizeof(addr_str));
+
 	    syslog(LOG_INFO|LOG_AUTH,
 		   "kerberos v5 shell from %s on %s as %s, cmd '%.80s'",
 		   name,
-		   inet_ntoa(thataddr.sin_addr),
+		   addr_str,
 		   server_username,
 		   cmd);
 	    free (name);
@@ -346,17 +406,19 @@ loop (int from0, int to0,
 	char buf[RSH_BUFSIZ];
 
 	ret = select (max_fd, &readset, NULL, NULL, NULL);
-	if (ret < 0)
+	if (ret < 0) {
 	    if (errno == EINTR)
 		continue;
 	    else
 		syslog_and_die ("select: %m");
+	}
 	if (FD_ISSET(from0, &readset)) {
 	    ret = do_read (from0, buf, sizeof(buf));
 	    if (ret < 0)
 		syslog_and_die ("read: %m");
 	    else if (ret == 0) {
 		close (from0);
+		close (to0);
 		FD_CLR(from0, &real_readset);
 	    } else
 		net_write (to0, buf, ret);
@@ -367,6 +429,7 @@ loop (int from0, int to0,
 		syslog_and_die ("read: %m");
 	    else if (ret == 0) {
 		close (from1);
+		close (to1);
 		FD_CLR(from1, &real_readset);
 		if (--count == 0)
 		    exit (0);
@@ -379,6 +442,7 @@ loop (int from0, int to0,
 		syslog_and_die ("read: %m");
 	    else if (ret == 0) {
 		close (from2);
+		close (to2);
 		FD_CLR(from2, &real_readset);
 		if (--count == 0)
 		    exit (0);
@@ -388,18 +452,35 @@ loop (int from0, int to0,
    }
 }
 
+/*
+ * Used by `setup_copier' to create some pipe-like means of
+ * communcation.  Real pipes would probably be the best thing, but
+ * then the shell doesn't understand it's talking to rshd.  If
+ * socketpair doesn't work everywhere, some autoconf magic would have
+ * to be added here.
+ *
+ * If it fails creating the `pipe', it aborts by calling fatal.
+ */
+
+static void
+pipe_a_like (int fd[2])
+{
+    if (socketpair (AF_UNIX, SOCK_STREAM, 0, fd) < 0)
+	fatal (STDOUT_FILENO, "socketpair: %m");
+}
+
+/*
+ * Start a child process and leave the parent copying data to and from it.  */
+
 static void
 setup_copier (void)
 {
     int p0[2], p1[2], p2[2];
     pid_t pid;
 
-    if (pipe(p0) < 0)
-	fatal (STDOUT_FILENO, "pipe: %m");
-    if (pipe(p1) < 0)
-	fatal (STDOUT_FILENO, "pipe: %m");
-    if (pipe(p2) < 0)
-	fatal (STDOUT_FILENO, "pipe: %m");
+    pipe_a_like(p0);
+    pipe_a_like(p1);
+    pipe_a_like(p2);
     pid = fork ();
     if (pid < 0)
 	fatal (STDOUT_FILENO, "fork: %m");
@@ -413,7 +494,7 @@ setup_copier (void)
 	close (p0[0]);
 	close (p1[1]);
 	close (p2[1]);
-    } else {
+    } else { /* parent */
 	close (p0[0]);
 	close (p1[1]);
 	close (p2[1]);
@@ -427,39 +508,62 @@ setup_copier (void)
     }
 }
 
+/*
+ * Is `port' a ``reserverd'' port?
+ */
+
 static int
 is_reserved(u_short port)
 {
     return ntohs(port) < IPPORT_RESERVED;
 }
 
+/*
+ * Set the necessary part of the environment in `env'.
+ */
+
+static void
+setup_environment (char *env[7], struct passwd *pwd)
+{
+    asprintf (&env[0], "USER=%s",  pwd->pw_name);
+    asprintf (&env[1], "HOME=%s",  pwd->pw_dir);
+    asprintf (&env[2], "SHELL=%s", pwd->pw_shell);
+    asprintf (&env[3], "PATH=%s",  _PATH_DEFPATH);
+    asprintf (&env[4], "SSH_CLIENT=only_to_make_bash_happy");
+    if (do_unique_tkfile)
+       asprintf (&env[5], "KRB5CCNAME=%s", tkfile);
+       else env[5] = NULL;
+    env[6] = NULL;
+}
+
 static void
 doit (int do_kerberos, int check_rhosts)
 {
     u_char buf[BUFSIZ];
     u_char *p;
-    struct sockaddr_in thisaddr, thataddr, erraddr;
+    struct sockaddr_storage thisaddr_ss;
+    struct sockaddr *thisaddr = (struct sockaddr *)&thisaddr_ss;
+    struct sockaddr_storage thataddr_ss;
+    struct sockaddr *thataddr = (struct sockaddr *)&thataddr_ss;
+    struct sockaddr_storage erraddr_ss;
+    struct sockaddr *erraddr = (struct sockaddr *)&erraddr_ss;
     int addrlen;
     int port;
     int errsock = -1;
-    char client_user[16], server_user[16];
+    char client_user[COMMAND_SZ], server_user[USERNAME_SZ];
     char cmd[COMMAND_SZ];
     struct passwd *pwd;
     int s = STDIN_FILENO;
-    char *env[5];
+    char *env[7];
 
-    addrlen = sizeof(thisaddr);
-    if (getsockname (s, (struct sockaddr *)&thisaddr, &addrlen) < 0
-	|| addrlen != sizeof(thisaddr)) {
+    addrlen = sizeof(thisaddr_ss);
+    if (getsockname (s, thisaddr, &addrlen) < 0)
 	syslog_and_die("getsockname: %m");
-    }
-    addrlen = sizeof(thataddr);
-    if (getpeername (s, (struct sockaddr *)&thataddr, &addrlen) < 0
-	|| addrlen != sizeof(thataddr)) {
+    addrlen = sizeof(thataddr_ss);
+    if (getpeername (s, thataddr, &addrlen) < 0)
 	syslog_and_die ("getpeername: %m");
-    }
 
-    if (!do_kerberos && !is_reserved(thataddr.sin_port))
+    if (!do_kerberos && !is_reserved(socket_get_port(thataddr)))
 	fatal(s, "Permission denied");
 
     p = buf;
@@ -487,14 +591,24 @@ doit (int do_kerberos, int check_rhosts)
 	 * do... :-(
 	 */
 
-	erraddr = thataddr;
-	erraddr.sin_port = htons(port);
-	errsock = rresvport (&priv_port);
+	erraddr->sa_family = thataddr->sa_family;
+	socket_set_address_and_port (erraddr,
+				     socket_get_address (thataddr),
+				     htons(port));
+
+	/*
+	 * we only do reserved port for IPv4
+	 */
+
+	if (erraddr->sa_family == AF_INET)
+	    errsock = rresvport (&priv_port);
+	else
+	    errsock = socket (erraddr->sa_family, SOCK_STREAM, 0);
 	if (errsock < 0)
 	    syslog_and_die ("socket: %m");
 	if (connect (errsock,
-		     (struct sockaddr *)&erraddr,
-		     sizeof(erraddr)) < 0)
+		     erraddr,
+		     socket_sockaddr_size (erraddr)) < 0)
 	    syslog_and_die ("connect: %m");
     }
     
@@ -519,12 +633,18 @@ doit (int do_kerberos, int check_rhosts)
 		syslog_and_die ("unrecognized auth protocol: %x %x %x %x",
 				buf[0], buf[1], buf[2], buf[3]);
     } else {
-	if(recv_bsd_auth (s, buf, thisaddr, thataddr,
+	if(recv_bsd_auth (s, buf,
+			  (struct sockaddr_in *)thisaddr,
+			  (struct sockaddr_in *)thataddr,
 			  client_user,
 			  server_user,
-			  cmd) == 0)
+			  cmd) == 0) {
 	    auth_method = AUTH_BROKEN;
-	else
+	    if(do_vacuous) {
+		printf("Remote host requires Kerberos authentication\n");
+		exit(0);
+	    }
+	} else
 	    syslog_and_die("recv_bsd_auth failed");
     }
 
@@ -537,6 +657,19 @@ doit (int do_kerberos, int check_rhosts)
 
     if (pwd->pw_uid != 0 && access (_PATH_NOLOGIN, F_OK) == 0)
 	fatal (s, "Login disabled.");
+
+#ifdef HAVE_GETSPNAM
+    {
+	struct spwd *sp;
+	long    today;
+    
+	sp = getspnam(server_user);
+	today = time(0)/(24L * 60 * 60);
+	if (sp->sp_expire > 0) 
+	    if (today > sp->sp_expire) 
+		fatal(s, "Account has expired.");
+    }
+#endif
     
 #ifdef HAVE_SETLOGIN
     if (setlogin(pwd->pw_name) < 0)
@@ -550,9 +683,30 @@ doit (int do_kerberos, int check_rhosts)
     if (initgroups (pwd->pw_name, pwd->pw_gid) < 0)
 	fatal (s, "Login incorrect.");
 
+    if (setgid(pwd->pw_gid) < 0)
+	fatal (s, "Login incorrect.");
+
     if (setuid (pwd->pw_uid) < 0)
 	fatal (s, "Login incorrect.");
 
+#ifdef KRB5
+    {
+	int fd;
+ 
+	if (!do_unique_tkfile)
+	    snprintf(tkfile,sizeof(tkfile),"FILE:/tmp/krb5cc_%u",pwd->pw_uid);
+	else if (*tkfile=='\0') {
+	    snprintf(tkfile,sizeof(tkfile),"FILE:/tmp/krb5cc_XXXXXX");
+	    fd = mkstemp(tkfile+5);
+	    close(fd);
+	    unlink(tkfile+5);
+	}
+ 
+	if (kerberos_status)
+	    krb5_start_session();
+    }
+#endif
+
     if (chdir (pwd->pw_dir) < 0)
 	fatal (s, "Remote directory.");
 
@@ -562,11 +716,7 @@ doit (int do_kerberos, int check_rhosts)
 	close (errsock);
     }
 
-    asprintf (&env[0], "USER=%s",  pwd->pw_name);
-    asprintf (&env[1], "HOME=%s",  pwd->pw_dir);
-    asprintf (&env[2], "SHELL=%s", pwd->pw_shell);
-    asprintf (&env[3], "PATH=%s",  _PATH_DEFPATH);
-    env[4] = NULL;
+    setup_environment (env, pwd);
 
     if (do_encrypt) {
 	setup_copier ();
@@ -575,54 +725,71 @@ doit (int do_kerberos, int check_rhosts)
 	    fatal (s, "write failed");
     }
 
+#ifdef KRB4
+    if(k_hasafs()) {
+	char cell[64];
+
+	if(do_newpag)
+	    k_setpag();
+	if (k_afs_cell_of_file (pwd->pw_dir, cell, sizeof(cell)) == 0)
+	    krb_afslog_uid_home (cell, NULL, pwd->pw_uid, pwd->pw_dir);
+
+	krb_afslog_uid_home(NULL, NULL, pwd->pw_uid, pwd->pw_dir);
+
+#ifdef KRB5
+	/* XXX */
+       {
+	   krb5_ccache ccache;
+	   krb5_error_code status;
+
+	   status = krb5_cc_resolve (context, tkfile, &ccache);
+	   if (!status) {
+	       krb5_afslog_uid_home(context,ccache,NULL,NULL,
+				    pwd->pw_uid, pwd->pw_dir);
+	       krb5_cc_close (context, ccache);
+	   }
+       }
+#endif /* KRB5 */
+    }
+#endif /* KRB4 */
     execle (pwd->pw_shell, pwd->pw_shell, "-c", cmd, NULL, env);
     err(1, "exec %s", pwd->pw_shell);
 }
 
-static void
-usage (int ret)
-{
-    syslog (LOG_ERR, "Usage: %s [-ixkl] [-p port]", __progname);
-    exit (ret);
-}
-
-static int do_inetd = 1;
-static char *port_str;
-static int do_rhosts;
-static int do_kerberos = 0;
-static int do_version;
-static int do_help = 0;
-
 struct getargs args[] = {
-    { "inetd",  'i', arg_negative_flag,	&do_inetd,
-      "Expect to be started by inetd",	NULL },
-    { "kerberos", 'k', arg_flag,	&do_kerberos,
-      "Implement kerberised services",	NULL },
-    { "encrypt", 'x', arg_flag,		&do_encrypt,
-      "Implement encrypted service",	NULL },
-    { "rhosts",	'l',	arg_negative_flag, &do_rhosts,
-      "Check users .rhosts",		NULL },
-    { "port",	'p', arg_string,	&port_str,	"Use this port",
-      "number-or-service" },
-    { "version", 0,  arg_flag,		&do_version,	"Print version",
-      NULL },
-    { "help",	 0,  arg_flag,		&do_help,	NULL,
-      NULL }
+    { "inetd",		'i',	arg_negative_flag,	&do_inetd,
+      "Not started from inetd" },
+    { "kerberos",	'k',	arg_flag,	&do_kerberos,
+      "Implement kerberised services" },
+    { "encrypt",	'x',	arg_flag,		&do_encrypt,
+      "Implement encrypted service" },
+    { "rhosts",		'l',	arg_flag, &do_rhosts,
+      "Check users .rhosts" },
+    { "port",		'p',	arg_string,	&port_str,	"Use this port",
+      "port" },
+    { "vacuous",	'v',	arg_flag, &do_vacuous,
+      "Don't accept non-kerberised connections" },
+    { NULL,		'P',	arg_negative_flag, &do_newpag,
+      "Don't put process in new PAG" },
+    /* compatibility flag: */
+    { NULL,		'L',	arg_flag, &do_log },
+    { "version",	0, 	arg_flag,		&do_version },
+    { "help",		0, 	arg_flag,		&do_help }
 };
 
-#if 0
-
 static void
 usage (int ret)
 {
-    arg_printusage (args,
-		    sizeof(args) / sizeof(args[0]),
-		    NULL,
-		    "host command");
+    if(isatty(STDIN_FILENO))
+	arg_printusage (args,
+			sizeof(args) / sizeof(args[0]),
+			NULL,
+			"");
+    else
+	syslog (LOG_ERR, "Usage: %s [-ikxlvPL] [-p port]", __progname);
     exit (ret);
 }
 
-#endif
 
 int
 main(int argc, char **argv)
@@ -641,10 +808,14 @@ main(int argc, char **argv)
 	usage (0);
 
     if (do_version) {
-	printf ("%s (%s-%s)\n", __progname, PACKAGE, VERSION);
-	return 0;
+	print_version(NULL);
+	exit(0);
     }
 
+#ifdef KRB5
+    krb5_init_context (&context);
+#endif
+
     if(port_str) {
 	struct servent *s = roken_getservbyname (port_str, "tcp");
 
