@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: commands.c,v 1.48 1999/02/01 03:08:27 assar Exp $");
+RCSID("$Id: commands.c,v 1.56 1999/09/16 20:41:35 assar Exp $");
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 int tos = -1;
@@ -67,7 +67,7 @@ makeargv()
     cp = line;
     if (*cp == '!') {		/* Special case shell escape */
 	/* save for shell command */
-	strcpy_truncate(saveline, line, sizeof(saveline));
+	strlcpy(saveline, line, sizeof(saveline));
 	*argp++ = "!";		/* No room in string to get this */
 	margc++;
 	cp++;
@@ -452,6 +452,19 @@ togdebug()
     return 1;
 }
 
+#if defined(KRB4) && defined(HAVE_KRB_DISABLE_DEBUG)
+#include <krb.h>
+
+static int
+togkrbdebug(void)
+{
+    if(krb_debug)
+	krb_enable_debug();
+    else
+	krb_disable_debug();
+    return 1;
+}
+#endif
 
 static int
 togcrlf()
@@ -662,6 +675,13 @@ static struct togglelist Togglelist[] = {
 	    togdebug,
 		&debug,
 		    "turn on socket level debugging" },
+#if defined(KRB4) && defined(HAVE_KRB_DISABLE_DEBUG)
+    { "krb_debug",
+      "kerberos 4 debugging",
+      togkrbdebug,
+      &krb_debug,
+      "turn on kerberos 4 debugging" },
+#endif
     { "netdata",
 	"printing of hexadecimal network data (debugging)",
 	    0,
@@ -1563,7 +1583,7 @@ env_init(void)
 		if (strchr(hbuf, '.') == 0) {
 			struct hostent *he = roken_gethostbyname(hbuf);
 			if (he != NULL)
-				strcpy_truncate(hbuf, he->h_name, 256);
+				strlcpy(hbuf, he->h_name, 256);
 		}
 
 		asprintf (&cp, "%s%s", hbuf, cp2);
@@ -1961,7 +1981,7 @@ cmdrc(char *m1, char *m2)
     if (skiprc)
 	return;
 
-    strcpy_truncate(m1save, m1, sizeof(m1save));
+    strlcpy(m1save, m1, sizeof(m1save));
     m1 = m1save;
 
     if (rcname[0] == 0) {
@@ -2055,7 +2075,7 @@ tn(int argc, char **argv)
 	return 0;
     }
     if (argc < 2) {
-	strcpy_truncate(line, "open ", sizeof(line));
+	strlcpy(line, "open ", sizeof(line));
 	printf("(to) ");
 	fgets(&line[strlen(line)], sizeof(line) - strlen(line), stdin);
 	makeargv();
@@ -2126,7 +2146,7 @@ tn(int argc, char **argv)
 	    sin6.sin6_family = family = AF_INET6;
 	    sa = (struct sockaddr *)&sin6;
 	    sa_size = sizeof(sin6);
-	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
+	    strlcpy(_hostname, hostp, sizeof(_hostname));
 	    hostname =_hostname;
 	} else
 #endif
@@ -2134,7 +2154,7 @@ tn(int argc, char **argv)
 	    sin.sin_family = family = AF_INET;
 	    sa = (struct sockaddr *)&sin;
 	    sa_size = sizeof(sin);
-	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
+	    strlcpy(_hostname, hostp, sizeof(_hostname));
 	    hostname = _hostname;
 	} else {
 #ifdef HAVE_GETHOSTBYNAME2
@@ -2147,7 +2167,7 @@ tn(int argc, char **argv)
 	    host = roken_gethostbyname(hostp);
 #endif
 	    if (host) {
-		strcpy_truncate(_hostname, host->h_name, sizeof(_hostname));
+		strlcpy(_hostname, host->h_name, sizeof(_hostname));
 		family = host->h_addrtype;
 		addr_list = host->h_addr_list;
 
@@ -2316,18 +2336,8 @@ tn(int argc, char **argv)
 #endif
     } while (connected == 0);
     cmdrc(hostp, hostname);
-    if (autologin && user == NULL) {
-	struct passwd *pw;
-
-	user = getenv("USER");
-	if (user == NULL ||
-	    ((pw = k_getpwnam((char *)user)) && pw->pw_uid != getuid())) {
-		if ((pw = k_getpwuid(getuid())))
-			user = pw->pw_name;
-		else
-			user = NULL;
-	}
-    }
+    if (autologin && user == NULL)
+	user = (char *)get_default_username ();
     if (user) {
 	env_define((unsigned char *)"USER", (unsigned char *)user);
 	env_export((unsigned char *)"USER");
@@ -2341,7 +2351,7 @@ tn(int argc, char **argv)
     return 0;
 }
 
-#define HELPINDENT (sizeof ("connect"))
+#define HELPINDENT ((int)sizeof ("connect"))
 
 static char
 	openhelp[] =	"connect to a site",
