@@ -33,7 +33,7 @@
 
 #include "telnetd.h"
 
-RCSID("$Id: telnetd.c,v 1.50 1998/06/09 19:24:48 joda Exp $");
+RCSID("$Id: telnetd.c,v 1.57 1999/09/16 20:41:37 assar Exp $");
 
 #ifdef _SC_CRAY_SECURE_SYS
 #include <sys/sysv.h>
@@ -117,7 +117,7 @@ int debug = 0;
 int keepalive = 1;
 char *progname;
 
-extern void usage (void);
+static void usage (void);
 
 /*
  * The string to pass to getopt().  We do it this way so
@@ -136,12 +136,14 @@ char valid_opts[] = "Bd:hklnS:u:UL:y"
 #endif
 		    ;
 
-void doit(struct sockaddr_in*);
+static void doit(struct sockaddr*, int);
 
-int main(int argc, char **argv)
+int
+main(int argc, char **argv)
 {
-    struct sockaddr_in from;
-    int on = 1, fromlen;
+    struct sockaddr_storage __ss;
+    struct sockaddr *sa = (struct sockaddr *)&__ss;
+    int on = 1, sa_size;
     int ch;
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
     int tos = -1;
@@ -366,10 +368,8 @@ int main(int argc, char **argv)
 
 	memset(&dv, 0, sizeof(dv));
 
-	if (getsysv(&sysv, sizeof(struct sysv)) != 0) {
-	    perror("getsysv");
-	    exit(1);
-	}
+	if (getsysv(&sysv, sizeof(struct sysv)) != 0) 
+	    fatalperror(net, "getsysv");
 
 	/*
 	 *	Get socket security label and set device values
@@ -379,10 +379,9 @@ int main(int argc, char **argv)
 	if ((getsockopt(0, SOL_SOCKET, SO_SECURITY,
 			(void *)&ss, &szss) < 0) ||
 	    (getsockopt(0, SOL_SOCKET, SO_SEC_MULTI,
-			(void *)&sock_multi, &szi) < 0)) {
-	    perror("getsockopt");
-	    exit(1);
-	} else {
+			(void *)&sock_multi, &szi) < 0)) 
+	    fatalperror(net, "getsockopt");
+	else {
 	    dv.dv_actlvl = ss.ss_actlabel.lt_level;
 	    dv.dv_actcmp = ss.ss_actlabel.lt_compart;
 	    if (!sock_multi) {
@@ -409,14 +408,14 @@ int main(int argc, char **argv)
 #endif	/* _SC_CRAY_SECURE_SYS */
 
     roken_openlog("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
-    fromlen = sizeof (from);
-    if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0) {
+    sa_size = sizeof (__ss);
+    if (getpeername(STDIN_FILENO, sa, &sa_size) < 0) {
 	fprintf(stderr, "%s: ", progname);
 	perror("getpeername");
 	_exit(1);
     }
     if (keepalive &&
-	setsockopt(0, SOL_SOCKET, SO_KEEPALIVE,
+	setsockopt(STDIN_FILENO, SOL_SOCKET, SO_KEEPALIVE,
 		   (void *)&on, sizeof (on)) < 0) {
 	syslog(LOG_WARNING, "setsockopt (SO_KEEPALIVE): %m");
     }
@@ -431,20 +430,21 @@ int main(int argc, char **argv)
 	if (tos < 0)
 	    tos = 020;	/* Low Delay bit */
 	if (tos
-	    && (setsockopt(0, IPPROTO_IP, IP_TOS,
+	    && sa->sa_family == AF_INET
+	    && (setsockopt(STDIN_FILENO, IPPROTO_IP, IP_TOS,
 			   (void *)&tos, sizeof(tos)) < 0)
 	    && (errno != ENOPROTOOPT) )
 	    syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
     }
 #endif	/* defined(IPPROTO_IP) && defined(IP_TOS) */
-    net = 0;
-    doit(&from);
+    net = STDIN_FILENO;
+    doit(sa, sa_size);
     /* NOTREACHED */
     return 0;
 }  /* end of main */
 
-void
-usage()
+static void
+usage(void)
 {
     fprintf(stderr, "Usage: telnetd");
 #ifdef	AUTHENTICATION
@@ -537,33 +537,33 @@ getterminaltype(char *name, size_t name_sz)
 	static unsigned char sb[] =
 	{ IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
 
-	net_write (sb, sizeof sb);
+	telnet_net_write (sb, sizeof sb);
 	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     if (his_state_is_will(TELOPT_XDISPLOC)) {
 	static unsigned char sb[] =
 	{ IAC, SB, TELOPT_XDISPLOC, TELQUAL_SEND, IAC, SE };
 
-	net_write (sb, sizeof sb);
+	telnet_net_write (sb, sizeof sb);
 	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     if (his_state_is_will(TELOPT_NEW_ENVIRON)) {
 	static unsigned char sb[] =
 	{ IAC, SB, TELOPT_NEW_ENVIRON, TELQUAL_SEND, IAC, SE };
 
-	net_write (sb, sizeof sb);
+	telnet_net_write (sb, sizeof sb);
 	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     else if (his_state_is_will(TELOPT_OLD_ENVIRON)) {
 	static unsigned char sb[] =
 	{ IAC, SB, TELOPT_OLD_ENVIRON, TELQUAL_SEND, IAC, SE };
 
-	net_write (sb, sizeof sb);
+	telnet_net_write (sb, sizeof sb);
 	DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
     }
     if (his_state_is_will(TELOPT_TTYPE)) {
 
-	net_write (ttytype_sbbuf, sizeof ttytype_sbbuf);
+	telnet_net_write (ttytype_sbbuf, sizeof ttytype_sbbuf);
 	DIAG(TD_OPTIONS, printsub('>', ttytype_sbbuf + 2,
 				  sizeof ttytype_sbbuf - 2););
     }
@@ -594,12 +594,12 @@ getterminaltype(char *name, size_t name_sz)
 	 * we have to just go with what we (might) have already gotten.
 	 */
 	if (his_state_is_will(TELOPT_TTYPE) && !terminaltypeok(terminaltype)) {
-	    strcpy_truncate(first, terminaltype, sizeof(first));
+	    strlcpy(first, terminaltype, sizeof(first));
 	    for(;;) {
 		/*
 		 * Save the unknown name, and request the next name.
 		 */
-		strcpy_truncate(last, terminaltype, sizeof(last));
+		strlcpy(last, terminaltype, sizeof(last));
 		_gettermname();
 		if (terminaltypeok(terminaltype))
 		    break;
@@ -638,7 +638,7 @@ _gettermname()
     if (his_state_is_wont(TELOPT_TTYPE))
 	return;
     settimer(baseline);
-    net_write (ttytype_sbbuf, sizeof ttytype_sbbuf);
+    telnet_net_write (ttytype_sbbuf, sizeof ttytype_sbbuf);
     DIAG(TD_OPTIONS, printsub('>', ttytype_sbbuf + 2,
 			      sizeof ttytype_sbbuf - 2););
     while (sequenceIs(ttypesubopt, baseline))
@@ -659,14 +659,20 @@ char remote_host_name[MaxHostNameLen];
 /*
  * Get a pty, scan input lines.
  */
-void
-doit(struct sockaddr_in *who)
+static void
+doit(struct sockaddr *who, int who_len)
 {
     char *host = NULL;
-    struct hostent *hp;
+    struct hostent *hp = NULL;
     int level;
     int ptynum;
     char user_name[256];
+    int error;
+    char host_addr[256];
+    void *addr;
+    int addr_sz;
+    const char *tmp;
+    int af;
 
     /*
      * Find an available pty to use.
@@ -691,34 +697,58 @@ doit(struct sockaddr_in *who)
     }
 #endif	/* _SC_CRAY_SECURE_SYS */
 
-    /* get name of connected client */
-    hp = roken_gethostbyaddr((const char *)&who->sin_addr,
-		       sizeof (struct in_addr),
-		       who->sin_family);
+    af = who->sa_family;
+    switch (af) {
+    case AF_INET : {
+	struct sockaddr_in *sin = (struct sockaddr_in *)who;
+
+	addr    = &sin->sin_addr;
+	addr_sz = sizeof(sin->sin_addr);
+	break;
+    }
+#ifdef HAVE_IPV6
+    case AF_INET6 : {
+	struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)who;
+
+	addr    = &sin6->sin6_addr;
+	addr_sz = sizeof(sin6->sin6_addr);
+	break;
+    }
+#endif
+    default :
+	fatal (net, "Unknown address family\r\n");
+	break;
+    }
+
+    hp = getipnodebyaddr (addr, addr_sz, af, &error);
 
     if (hp == NULL && registerd_host_only) {
 	fatal(net, "Couldn't resolve your address into a host name.\r\n\
 Please contact your net administrator");
-    } else if (hp) {
+    } else if (hp != NULL) {
 	host = hp->h_name;
-    } else {
-	host = inet_ntoa(who->sin_addr);
     }
+
+    tmp = inet_ntop(af, addr, host_addr, sizeof(host_addr));
+    if (tmp == NULL)
+	strlcpy (host_addr, "unknown address", sizeof(host_addr));
+
+    if (host == NULL)
+	host = host_addr;
+
     /*
      * We must make a copy because Kerberos is probably going
      * to also do a gethost* and overwrite the static data...
      */
-    strcpy_truncate(remote_host_name, host, sizeof(remote_host_name));
+    strlcpy(remote_host_name, host, sizeof(remote_host_name));
+    if (hp != NULL)
+	freehostent (hp);
     host = remote_host_name;
 
     /* XXX - should be k_gethostname? */
     gethostname(host_name, sizeof (host_name));
     hostname = host_name;
 
-#ifndef abs
-#define abs(x) ((x < 0) ? (-x) : x)
-#endif
-
     /* Only trim if too long (and possible) */
     if (strlen(remote_host_name) > abs(utmp_len)) {
 	char *domain = strchr(host_name, '.');
@@ -732,9 +762,9 @@ Please contact your net administrator");
      * If hostname still doesn't fit utmp, use ipaddr.
      */
     if (strlen(remote_host_name) > abs(utmp_len))
-	strcpy_truncate(remote_host_name,
-		inet_ntoa(who->sin_addr),
-		sizeof(remote_host_name));
+	strlcpy(remote_host_name,
+			host_addr,
+			sizeof(remote_host_name));
 
 #ifdef AUTHENTICATION
     auth_encrypt_init(hostname, host, "TELNETD", 1);
