@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,7 +38,7 @@
 
 #include "kuser_locl.h"
 
-RCSID("$Id: klist.c,v 1.39 1999/02/11 21:03:08 joda Exp $");
+RCSID("$Id: klist.c,v 1.50 1999/09/24 12:53:41 joda Exp $");
 
 static char*
 printable_time(time_t t)
@@ -103,10 +103,15 @@ print_cred_verbose(krb5_context context, krb5_creds *cred)
 	Ticket t;
 	size_t len;
 	char *s;
+
 	decode_Ticket(cred->ticket.data, cred->ticket.length, &t, &len);
-	krb5_enctype_to_string(context, t.enc_part.etype, &s);
-	printf("Ticket etype: %s", s);
-	free(s);
+	ret = krb5_enctype_to_string(context, t.enc_part.etype, &s);
+	if (ret == 0) {
+	    printf("Ticket etype: %s", s);
+	    free(s);
+	} else {
+	    printf("Unknown etype: %d", t.enc_part.etype);
+	}
 	if(t.enc_part.kvno)
 	    printf(", kvno %d", *t.enc_part.kvno);
 	printf("\n");
@@ -155,40 +160,14 @@ print_cred_verbose(krb5_context context, krb5_creds *cred)
     printf("\n");
     printf("Addresses: ");
     for(j = 0; j < cred->addresses.len; j++){
+	char buf[128];
+	size_t len;
 	if(j) printf(", ");
-	switch(cred->addresses.val[j].addr_type){
-	case KRB5_ADDRESS_INET : {
-	    struct in_addr a;
-	    unsigned long foo;
+	ret = krb5_print_address(&cred->addresses.val[j], 
+				 buf, sizeof(buf), &len);
 
-	    k_get_int (cred->addresses.val[j].address.data,
-		       &foo, 4);
-	    a.s_addr = htonl(foo);
-	    
-	    printf("IPv4: %s", inet_ntoa(a));
-	    break;
-	}
-#if defined(AF_INET6) && defined(HAVE_INET_NTOP) && defined(INET6_ADDRSTRLEN)
-	case KRB5_ADDRESS_INET6: {
-	    char foo[INET6_ADDRSTRLEN];
-
-	    printf("IPv6: %s", inet_ntop(AF_INET6,
-					 cred->addresses.val[j].address.data,
-					 foo, sizeof(foo)));
-	    break;
-	}
-#endif
-	default:{
-	    char *s;
-	    int i;
-	    krb5_address *a = &cred->addresses.val[j];
-	    s = malloc(a->address.length * 2 + 1);
-	    for(i = 0; i < a->address.length; i++)
-		sprintf(s + 2*i, "%02x", ((char*)a->address.data)[i]);
-	    printf("%d/%s", cred->addresses.val[j].addr_type, s);
-	    free(s);
-	}
-	}
+	if(ret == 0)
+	    printf("%s", buf);
     }
     printf("\n\n");
 }
@@ -212,10 +191,17 @@ print_tickets (krb5_context context,
     if (ret)
 	krb5_err (context, 1, ret, "krb5_unparse_name");
 
-    printf ("Credentials cache: %s\n", krb5_cc_get_name(context, ccache));
-    printf ("\tPrincipal: %s\n\n", str);
+    printf ("%17s: %s:%s\n", 
+	    "Credentials cache",
+	    krb5_cc_get_type(context, ccache),
+	    krb5_cc_get_name(context, ccache));
+    printf ("%17s: %s\n", "Principal", str);
     free (str);
-
+    
+    if(do_verbose)
+	printf ("%17s: %d\n", "Cache version",
+		krb5_cc_get_version(context, ccache));
+    
     if (do_verbose && context->kdc_sec_offset) {
 	char buf[BUFSIZ];
 	int val;
@@ -230,10 +216,12 @@ print_tickets (krb5_context context,
 
 	unparse_time (val, buf, sizeof(buf));
 
-	printf ("\tKDC time offset: %s%s\n",
+	printf ("%17s: %s%s\n", "KDC time offset",
 		sig == -1 ? "-" : "", buf);
     }
 
+    printf("\n");
+
     ret = krb5_cc_start_seq_get (context, ccache, &cursor);
     if (ret)
 	krb5_err(context, 1, ret, "krb5_cc_start_seq_get");
@@ -293,14 +281,82 @@ check_for_tgt (krb5_context context,
     return expired;
 }
 
+#ifdef KRB4
+/*
+ * Print a list of all AFS tokens
+ */
+
+static void
+display_tokens(int do_verbose)
+{
+    u_int32_t i;
+    unsigned char t[128];
+    struct ViceIoctl parms;
+
+    parms.in = (void *)&i;
+    parms.in_size = sizeof(i);
+    parms.out = (void *)t;
+    parms.out_size = sizeof(t);
+
+    for (i = 0; k_pioctl(NULL, VIOCGETTOK, &parms, 0) == 0; i++) {
+        int32_t size_secret_tok, size_public_tok;
+        char *cell;
+	struct ClearToken ct;
+	unsigned char *r = t;
+	struct timeval tv;
+	char buf1[20], buf2[20];
+
+	memcpy(&size_secret_tok, r, sizeof(size_secret_tok));
+	/* dont bother about the secret token */
+	r += size_secret_tok + sizeof(size_secret_tok);
+	memcpy(&size_public_tok, r, sizeof(size_public_tok));
+	r += sizeof(size_public_tok);
+	memcpy(&ct, r, size_public_tok);
+	r += size_public_tok;
+	/* there is a int32_t with length of cellname, but we dont read it */
+	r += sizeof(int32_t);
+	cell = r;
+
+	gettimeofday (&tv, NULL);
+	strlcpy (buf1, printable_time(ct.BeginTimestamp),
+			 sizeof(buf1));
+	if (do_verbose || tv.tv_sec < ct.EndTimestamp)
+	    strlcpy (buf2, printable_time(ct.EndTimestamp),
+			     sizeof(buf2));
+	else
+	    strlcpy (buf2, ">>> Expired <<<", sizeof(buf2));
+
+	printf("%s  %s  ", buf1, buf2);
+
+	if ((ct.EndTimestamp - ct.BeginTimestamp) & 1)
+	  printf("User's (AFS ID %d) tokens for %s", ct.ViceId, cell);
+	else
+	  printf("Tokens for %s", cell);
+	if (do_verbose)
+	    printf(" (%d)", ct.AuthHandle);
+	putchar('\n');
+    }
+}
+#endif
+
 static int version_flag = 0;
 static int help_flag	= 0;
 static int do_verbose	= 0;
 static int do_test	= 0;
+#ifdef KRB4
+static int do_tokens	= 0;
+#endif
+static char *cred_cache;
 
 static struct getargs args[] = {
+    { "cache",			'c', arg_string, &cred_cache,
+      "credentials cache to list", "cache" },
     { "test",			't', arg_flag, &do_test,
       "test for having tickets", NULL },
+#ifdef KRB4
+    { "tokens",			'T',   arg_flag, &do_tokens,
+      "display AFS tokens", NULL },
+#endif
     { "verbose",		'v', arg_flag, &do_verbose,
       "Verbose output", NULL },
     { "version", 		0,   arg_flag, &version_flag, 
@@ -338,7 +394,7 @@ main (int argc, char **argv)
 	usage (0);
 
     if(version_flag){
-	printf("%s (%s-%s)\n", __progname, PACKAGE, VERSION);
+	print_version(NULL);
 	exit(0);
     }
 
@@ -352,17 +408,27 @@ main (int argc, char **argv)
     if (ret)
 	krb5_err(context, 1, ret, "krb5_init_context");
 
-    ret = krb5_cc_default (context, &ccache);
-    if (ret)
-	krb5_err (context, 1, ret, "krb5_cc_default");
+    if(cred_cache) {
+	ret = krb5_cc_resolve(context, cred_cache, &ccache);
+	if (ret)
+	    krb5_err (context, 1, ret, "%s", cred_cache);
+    } else {
+	ret = krb5_cc_default (context, &ccache);
+	if (ret)
+	    krb5_err (context, 1, ret, "krb5_cc_resolve");
+    }
 
     ret = krb5_cc_get_principal (context, ccache, &principal);
-    if(ret == ENOENT)
-	krb5_errx(context, 1, "No ticket file: %s", 
-		  krb5_cc_get_name(context, ccache));
-    if (ret)
-	krb5_err (context, 1, ret, "krb5_cc_get_principal");
-
+    if (ret) {
+	if(ret == ENOENT) {
+	    if (do_test)
+		return 1;
+	    else
+		krb5_errx(context, 1, "No ticket file: %s", 
+			  krb5_cc_get_name(context, ccache));
+	} else
+	    krb5_err (context, 1, ret, "krb5_cc_get_principal");
+    }
     if (do_test)
 	exit_status = check_for_tgt (context, ccache, principal);
     else
@@ -374,5 +440,11 @@ main (int argc, char **argv)
 
     krb5_free_principal (context, principal);
     krb5_free_context (context);
+
+#ifdef KRB4
+    if (!do_test && do_tokens && k_hasafs ())
+	display_tokens (do_verbose);
+#endif
+
     return exit_status;
 }
