@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,24 +38,21 @@
 
 #include "der_locl.h"
 
-RCSID("$Id: der_get.c,v 1.17 1997/07/16 21:28:46 joda Exp $");
+RCSID("$Id: der_get.c,v 1.26 1999/10/10 04:48:58 assar Exp $");
 
-/*
- * All decoding functions take a pointer `p' to first position in
- * which to read, from the left, `len' which means the maximum
- * number of characters we are able to read and return an int
- * indicating how many actually got read, or <0 in case of errors.
- */
+#include <version.h>
 
 /* 
  * All decoding functions take a pointer `p' to first position in
  * which to read, from the left, `len' which means the maximum number
- * of characters we are able to read and return the status as an int
+ * of characters we are able to read, `ret' were the value will be
+ * returned and `size' where the number of used bytes is stored.
+ * Either 0 or an error code is returned.
  */
 
-
-int
-der_get_int (unsigned char *p, size_t len, unsigned *ret, size_t *size)
+static int
+der_get_unsigned (const unsigned char *p, size_t len,
+		  unsigned *ret, size_t *size)
 {
     unsigned val = 0;
     size_t oldlen = len;
@@ -63,98 +60,106 @@ der_get_int (unsigned char *p, size_t len, unsigned *ret, size_t *size)
     while (len--)
 	val = val * 256 + *p++;
     *ret = val;
-    *size = oldlen;
+    if(size) *size = oldlen;
     return 0;
 }
 
 int
-der_get_length (unsigned char *p, size_t len, size_t *val, size_t *size)
+der_get_int (const unsigned char *p, size_t len,
+	     int *ret, size_t *size)
+{
+    int val = 0;
+    size_t oldlen = len;
+
+    if (len--)
+	val = (signed char)*p++;
+    while (len--)
+	val = val * 256 + *p++;
+    *ret = val;
+    if(size) *size = oldlen;
+    return 0;
+}
+
+int
+der_get_length (const unsigned char *p, size_t len,
+		size_t *val, size_t *size)
 {
     size_t v;
 
-    if (--len < 0)
+    if (len <= 0)
 	return ASN1_OVERRUN;
+    --len;
     v = *p++;
     if (v < 128) {
 	*val = v;
-	*size = 1;
+	if(size) *size = 1;
     } else {
 	int e;
 	size_t l;
 	unsigned tmp;
 
+	if(v == 0x80){
+	    *val = ASN1_INDEFINITE;
+	    if(size) *size = 1;
+	    return 0;
+	}
 	v &= 0x7F;
 	if (len < v)
 	    return ASN1_OVERRUN;
-	e = der_get_int (p, v, &tmp, &l);
+	e = der_get_unsigned (p, v, &tmp, &l);
 	if(e) return e;
 	*val = tmp;
-	*size = l + 1;
+	if(size) *size = l + 1;
     }
     return 0;
 }
 
 int
-der_get_general_string (unsigned char *p, size_t len, 
+der_get_general_string (const unsigned char *p, size_t len, 
 			general_string *str, size_t *size)
 {
-    size_t l, slen;
     char *s;
-    int e;
 
-    e = der_get_length (p, len, &slen, &l);
-    if (e) return e;
-    p += l;
-    len -= l;
-    if (len < slen)
-	return ASN1_OVERRUN;
-    s = malloc (slen + 1);
+    s = malloc (len + 1);
     if (s == NULL)
 	return ENOMEM;
-    memcpy (s, p, slen);
-    s[slen] = '\0';
+    memcpy (s, p, len);
+    s[len] = '\0';
     *str = s;
-    *size = slen + l;
+    if(size) *size = len;
     return 0;
 }
 
 int
-der_get_octet_string (unsigned char *p, size_t len, 
+der_get_octet_string (const unsigned char *p, size_t len, 
 		      octet_string *data, size_t *size)
 {
-    size_t l, slen;
-    int e;
-    
-    e = der_get_length (p, len, &slen, &l);
-    if (e) return e;
-    p += l;
-    len -= l;
-    if (len < slen)
-	return ASN1_OVERRUN;
-    data->length = slen;
-    data->data = malloc(slen);
+    data->length = len;
+    data->data = malloc(len);
     if (data->data == NULL && data->length != 0)
 	return ENOMEM;
-    memcpy (data->data, p, slen);
-    *size = slen + l;
+    memcpy (data->data, p, len);
+    if(size) *size = len;
     return 0;
 }
 
 int
-der_get_tag (unsigned char *p, size_t len, Der_class *class, Der_type *type,
+der_get_tag (const unsigned char *p, size_t len,
+	     Der_class *class, Der_type *type,
 	     int *tag, size_t *size)
 {
     if (len < 1)
 	return ASN1_OVERRUN;
-    *class = ((*p) >> 6) & 0x03;
-    *type = ((*p) >> 5) & 0x01;
+    *class = (Der_class)(((*p) >> 6) & 0x03);
+    *type = (Der_type)(((*p) >> 5) & 0x01);
     *tag = (*p) & 0x1F;
-    *size = 1;
+    if(size) *size = 1;
     return 0;
 }
 
 int
-der_match_tag (unsigned char *p, size_t len, Der_class class, Der_type type,
+der_match_tag (const unsigned char *p, size_t len,
+	       Der_class class, Der_type type,
 	       int tag, size_t *size)
 {
     size_t l;
@@ -171,12 +176,12 @@ der_match_tag (unsigned char *p, size_t len, Der_class class, Der_type type,
 	return ASN1_MISPLACED_FIELD;
     if(tag < thistag)
 	return ASN1_MISSING_FIELD;
-    *size = l;
+    if(size) *size = l;
     return 0;
 }
 
 int
-der_match_tag_and_length (unsigned char *p, size_t len,
+der_match_tag_and_length (const unsigned char *p, size_t len,
 			  Der_class class, Der_type type, int tag,
 			  size_t *length_ret, size_t *size)
 {
@@ -193,12 +198,13 @@ der_match_tag_and_length (unsigned char *p, size_t len,
     p += l;
     len -= l;
     ret += l;
-    *size = ret;
+    if(size) *size = ret;
     return 0;
 }
 
 int
-decode_integer (unsigned char *p, size_t len, unsigned *num, size_t *size)
+decode_integer (const unsigned char *p, size_t len,
+		int *num, size_t *size)
 {
     size_t ret = 0;
     size_t l, reallen;
@@ -219,59 +225,80 @@ decode_integer (unsigned char *p, size_t len, unsigned *num, size_t *size)
     p += l;
     len -= l;
     ret += l;
-    *size = ret;
+    if(size) *size = ret;
     return 0;
 }
 
 int
-decode_general_string (unsigned char *p, size_t len, 
+decode_general_string (const unsigned char *p, size_t len, 
 		       general_string *str, size_t *size)
 {
     size_t ret = 0;
     size_t l;
     int e;
+    size_t slen;
 
     e = der_match_tag (p, len, UNIV, PRIM, UT_GeneralString, &l);
     if (e) return e;
     p += l;
     len -= l;
     ret += l;
-    e = der_get_general_string (p, len, str, &l);
+
+    e = der_get_length (p, len, &slen, &l);
     if (e) return e;
     p += l;
     len -= l;
     ret += l;
-    *size = ret;
+    if (len < slen)
+	return ASN1_OVERRUN;
+
+    e = der_get_general_string (p, slen, str, &l);
+    if (e) return e;
+    p += l;
+    len -= l;
+    ret += l;
+    if(size) *size = ret;
     return 0;
 }
 
 int
-decode_octet_string (unsigned char *p, size_t len, 
+decode_octet_string (const unsigned char *p, size_t len, 
 		     octet_string *k, size_t *size)
 {
     size_t ret = 0;
     size_t l;
     int e;
+    size_t slen;
 
     e = der_match_tag (p, len, UNIV, PRIM, UT_OctetString, &l);
     if (e) return e;
     p += l;
     len -= l;
     ret += l;
-    e = der_get_octet_string (p, len, k, &l);
+
+    e = der_get_length (p, len, &slen, &l);
     if (e) return e;
     p += l;
     len -= l;
     ret += l;
-    *size = ret;
+    if (len < slen)
+	return ASN1_OVERRUN;
+
+    e = der_get_octet_string (p, slen, k, &l);
+    if (e) return e;
+    p += l;
+    len -= l;
+    ret += l;
+    if(size) *size = ret;
     return 0;
 }
 
 static void
-generalizedtime2time (char *s, time_t *t)
+generalizedtime2time (const char *s, time_t *t)
 {
     struct tm tm;
 
+    memset(&tm, 0, sizeof(tm));
     sscanf (s, "%04d%02d%02d%02d%02d%02dZ",
 	    &tm.tm_year, &tm.tm_mon, &tm.tm_mday, &tm.tm_hour,
 	    &tm.tm_min, &tm.tm_sec);
@@ -281,20 +308,30 @@ generalizedtime2time (char *s, time_t *t)
 }
 
 int
-decode_generalized_time (unsigned char *p, size_t len, time_t *t, size_t *size)
+decode_generalized_time (const unsigned char *p, size_t len,
+			 time_t *t, size_t *size)
 {
     octet_string k;
     char *times;
     size_t ret = 0;
     size_t l;
     int e;
+    size_t slen;
 
     e = der_match_tag (p, len, UNIV, PRIM, UT_GeneralizedTime, &l);
     if (e) return e;
     p += l;
     len -= l;
     ret += l;
-    e = der_get_octet_string (p, len, &k, &l);
+
+    e = der_get_length (p, len, &slen, &l);
+    if (e) return e;
+    p += l;
+    len -= l;
+    ret += l;
+    if (len < slen)
+	return ASN1_OVERRUN;
+    e = der_get_octet_string (p, slen, &k, &l);
     if (e) return e;
     p += l;
     len -= l;
@@ -307,7 +344,7 @@ decode_generalized_time (unsigned char *p, size_t len, time_t *t, size_t *size)
     times[k.length] = 0;
     generalizedtime2time (times, t);
     free (times);
-    *size = ret;
+    if(size) *size = ret;
     return 0;
 }
 
@@ -315,7 +352,7 @@ decode_generalized_time (unsigned char *p, size_t len, time_t *t, size_t *size)
 int
 fix_dce(size_t reallen, size_t *len)
 {
-    if(reallen == 0)
+    if(reallen == ASN1_INDEFINITE)
 	return 1;
     if(*len < reallen)
 	return -1;
