@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -36,7 +36,7 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: hdb_locl.h,v 1.2 1997/07/16 21:35:35 joda Exp $ */
+/* $Id: hdb_locl.h,v 1.11 1999/05/03 16:55:14 joda Exp $ */
 
 #ifndef __HDB_LOCL_H__
 #define __HDB_LOCL_H__
@@ -47,11 +47,27 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+#ifdef HAVE_SYS_FILE_H
+#include <sys/file.h>
+#endif
+#include <roken.h>
 
 #include <krb5.h>
 #include <hdb.h>
+#include <hdb-private.h>
 
-#ifdef HAVE_DB_H
+#if defined(HAVE_DB_185_H)
+#include <db_185.h>
+#elif defined(HAVE_DB_H)
 #include <db.h>
 #endif
 
@@ -59,9 +75,14 @@
 #include <ndbm.h>
 #endif
 
-void hdb_principal2key(krb5_context, krb5_principal, krb5_data*);
-void hdb_key2principal(krb5_context, krb5_data*, krb5_principal*);
-void hdb_entry2value(krb5_context, hdb_entry*, krb5_data*);
-void hdb_value2entry(krb5_context, krb5_data*, hdb_entry*);
+int hdb_principal2key(krb5_context, krb5_principal, krb5_data*);
+int hdb_key2principal(krb5_context, krb5_data*, krb5_principal);
+
+krb5_error_code hdb_lock(int, int);
+krb5_error_code hdb_unlock(int);
+
+krb5_error_code _hdb_fetch(krb5_context, HDB*, unsigned, hdb_entry*);
+krb5_error_code _hdb_store(krb5_context, HDB*, unsigned, hdb_entry*);
+krb5_error_code _hdb_remove(krb5_context, HDB*, hdb_entry*);
 
 #endif /* __HDB_LOCL_H__ */
