@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,17 +38,107 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: context.c,v 1.39 1999/02/11 21:03:23 joda Exp $");
+RCSID("$Id: context.c,v 1.47 1999/09/28 02:25:00 assar Exp $");
 
-int issuid(void); /* XXX */
+#define INIT_FIELD(C, T, E, D, F)					\
+    (C)->E = krb5_config_get_ ## T ## _default ((C), NULL, (D), 	\
+						"libdefaults", F, NULL)
+
+#ifdef KRB4
+extern krb5_kt_ops krb4_fkt_ops;
+#endif
+
+static krb5_error_code
+init_context_from_config_file(krb5_context context)
+{
+    const char * tmp;
+    INIT_FIELD(context, time, max_skew, 5 * 60, "clockskew");
+    INIT_FIELD(context, time, kdc_timeout, 3, "kdc_timeout");
+    INIT_FIELD(context, int, max_retries, 3, "max_retries");
+
+    context->http_proxy = krb5_config_get_string(context, NULL, "libdefaults", 
+					   "http_proxy", NULL);
+
+    {
+	char **etypes;
+	etypes = krb5_config_get_strings(context, NULL, "libdefaults", 
+					 "default_etypes", NULL);
+	if(etypes){
+	    int i, j, k;
+	    for(i = 0; etypes[i]; i++);
+	    context->etypes = malloc((i+1) * sizeof(*context->etypes));
+	    for(j = 0, k = 0; j < i; j++) {
+		if(krb5_string_to_enctype(context, etypes[j], &context->etypes[k]) == 0)
+		    k++;
+	    }
+	    context->etypes[k] = ETYPE_NULL;
+	    krb5_config_free_strings(etypes);
+	}
+    }
+    /* default keytab name */
+    context->default_keytab = krb5_config_get_string(context, NULL, 
+					       "libdefaults", 
+					       "default_keytab_name", 
+					       NULL);
+    if(context->default_keytab == NULL)
+	context->default_keytab = KEYTAB_DEFAULT;
+
+    context->time_fmt = krb5_config_get_string(context, NULL, "libdefaults", 
+					 "time_format", NULL);
+    if(context->time_fmt == NULL)
+	context->time_fmt = "%d-%b-%Y %H:%M:%S";
+    context->log_utc = krb5_config_get_bool(context, NULL, "libdefaults",
+					    "log_utc", NULL);
+
+    /* init dns-proxy slime */
+    tmp = krb5_config_get_string(context, NULL, "libdefaults", 
+				 "dns_proxy", NULL);
+    if(tmp) 
+	roken_gethostby_setup(context->http_proxy, tmp);
+    krb5_set_default_realm(context, NULL);
+
+    {
+	krb5_addresses addresses;
+	char **adr, **a;
+	adr = krb5_config_get_strings(context, NULL, 
+				      "libdefaults", 
+				      "extra_addresses", 
+				      NULL);
+	memset(&addresses, 0, sizeof(addresses));
+	for(a = adr; a && *a; a++) {
+	    krb5_parse_address(context, *a, &addresses);
+	    krb5_add_extra_addresses(context, &addresses);
+	    krb5_free_addresses(context, &addresses);
+	}
+	krb5_config_free_strings(adr);
+    }
+    
+    INIT_FIELD(context, bool, srv_lookup, TRUE, "srv_lookup");
+    INIT_FIELD(context, bool, srv_try_txt, FALSE, "srv_try_txt");
+    INIT_FIELD(context, bool, srv_try_rfc2052, TRUE, "srv_try_rfc2052");
+    INIT_FIELD(context, int, fcache_vno, 0, "fcache_version");
+
+    context->cc_ops       = NULL;
+    context->num_cc_ops	  = 0;
+    krb5_cc_register(context, &krb5_fcc_ops, TRUE);
+    krb5_cc_register(context, &krb5_mcc_ops, TRUE);
+
+    context->num_kt_types = 0;
+    context->kt_types     = NULL;
+    krb5_kt_register (context, &krb5_fkt_ops);
+    krb5_kt_register (context, &krb5_mkt_ops);
+#ifdef KRB4
+    krb5_kt_register (context, &krb4_fkt_ops);
+#endif
+    krb5_kt_register (context, &krb5_akf_ops);
+    return 0;
+}
 
 krb5_error_code
 krb5_init_context(krb5_context *context)
 {
     krb5_context p;
-    int val;
     const char *config_file = NULL;
-    const char * tmp;
     krb5_config_section *tmp_cf;
     krb5_error_code ret;
 
@@ -69,64 +159,16 @@ krb5_init_context(krb5_context *context)
 
     if (ret == 0)
 	p->cf = tmp_cf;
+#if 0
     else
 	krb5_warnx (p, "Unable to parse config file %s.  Ignoring.",
 		    config_file); /* XXX */
+#endif
 
-    p->max_skew = 5 * 60;
-    val = krb5_config_get_time (p, NULL, "libdefaults", "clockskew", NULL);
-    if (val >= 0)
-	p->max_skew = val;
+    ret = init_context_from_config_file(p);
+    if(ret)
+	return ret;
 
-    p->kdc_timeout = 3;
-    val = krb5_config_get_time (p, NULL, "libdefaults", "kdc_timeout", NULL);
-    if(val >= 0) 
-	p->kdc_timeout = val;
-
-    p->max_retries = 3;
-    val = krb5_config_get_int (p, NULL, "libdefaults", "max_retries", NULL);
-    if (val >= 0)
-	p->max_retries = val;
-
-    p->http_proxy = krb5_config_get_string(p, NULL, "libdefaults", 
-					   "http_proxy", NULL);
-
-    {
-	char **etypes;
-	etypes = krb5_config_get_strings(p, NULL, "libdefaults", 
-					 "default_etypes", NULL);
-	if(etypes){
-	    int i, j, k;
-	    for(i = 0; etypes[i]; i++);
-	    p->etypes = malloc((i+1) * sizeof(*p->etypes));
-	    for(j = 0, k = 0; j < i; j++) {
-		if(krb5_string_to_enctype(p, etypes[j], &p->etypes[k]) == 0)
-		    k++;
-	    }
-	    p->etypes[k] = ETYPE_NULL;
-	    krb5_config_free_strings(etypes);
-	}
-    }
-    /* default keytab name */
-    p->default_keytab = krb5_config_get_string(p, NULL, 
-					       "libdefaults", 
-					       "default_keytab_name", 
-					       NULL);
-    if(p->default_keytab == NULL)
-	p->default_keytab = KEYTAB_DEFAULT;
-
-    p->time_fmt = krb5_config_get_string(p, NULL, "libdefaults", 
-					 "time_format", NULL);
-    if(p->time_fmt == NULL)
-	p->time_fmt = "%d-%b-%Y %H:%M:%S";
-    p->log_utc = krb5_config_get_bool(p, NULL, "libdefaults", "log_utc", NULL);
-
-    /* init dns-proxy slime */
-    tmp = krb5_config_get_string(p, NULL, "libdefaults", 
-				 "dns_proxy", NULL);
-    if(tmp) 
-	roken_gethostby_setup(p->http_proxy, tmp);
-    krb5_set_default_realm(p, NULL);
     *context = p;
     return 0;
 }
@@ -137,12 +179,13 @@ krb5_free_context(krb5_context context)
   int i;
 
   free(context->etypes);
-  free(context->default_realm);
-  krb5_config_file_free (context->cf);
+  krb5_free_host_realm (context, context->default_realms);
+  krb5_config_file_free (context, context->cf);
   free_error_table (context->et_list);
-  for(i = 0; i < context->num_ops; ++i)
+  for(i = 0; i < context->num_cc_ops; ++i)
     free(context->cc_ops[i].prefix);
   free(context->cc_ops);
+  free(context->kt_types);
   free(context);
 }
 
@@ -212,7 +255,7 @@ krb5_get_default_in_tkt_etypes(krb5_context context,
 }
 
 const char *
-krb5_get_err_text(krb5_context context, long code)
+krb5_get_err_text(krb5_context context, krb5_error_code code)
 {
     const char *p = com_right(context->et_list, code);
     if(p == NULL)
@@ -241,3 +284,53 @@ krb5_get_use_admin_kdc (krb5_context context)
 {
     return context->use_admin_kdc;
 }
+
+krb5_error_code
+krb5_add_extra_addresses(krb5_context context, krb5_addresses *addresses)
+{
+
+    if(context->extra_addresses)
+	return krb5_append_addresses(context, 
+				     context->extra_addresses, addresses);
+    else
+	return krb5_set_extra_addresses(context, addresses);
+}
+
+krb5_error_code
+krb5_set_extra_addresses(krb5_context context, krb5_addresses *addresses)
+{
+    if(context->extra_addresses) {
+	krb5_free_addresses(context, context->extra_addresses);
+	free(context->extra_addresses);
+    }
+    if(context->extra_addresses == NULL) {
+	context->extra_addresses = malloc(sizeof(*context->extra_addresses));
+	if(context->extra_addresses == NULL)
+	    return ENOMEM;
+    }
+    return copy_HostAddresses(addresses, context->extra_addresses);
+}
+
+krb5_error_code
+krb5_get_extra_addresses(krb5_context context, krb5_addresses *addresses)
+{
+    if(context->extra_addresses == NULL) {
+	memset(addresses, 0, sizeof(*addresses));
+	return 0;
+    }
+    return copy_HostAddresses(context->extra_addresses, addresses);
+}
+
+krb5_error_code
+krb5_set_fcache_version(krb5_context context, int version)
+{
+    context->fcache_vno = version;
+    return 0;
+}
+
+krb5_error_code
+krb5_get_fcache_version(krb5_context context, int *version)
+{
+    *version = context->fcache_vno;
+    return 0;
+}
