@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -37,13 +37,15 @@
  */
 
 #include "krb5_locl.h"
+#ifdef HAVE_RES_SEARCH
 #define USE_RESOLVER
+#endif
 #ifdef HAVE_ARPA_NAMESER_H
 #include <arpa/nameser.h>
 #endif
 #include "resolve.h"
 
-RCSID("$Id: principal.c,v 1.42 1999/02/11 21:03:47 joda Exp $");
+RCSID("$Id: principal.c,v 1.49 1999/08/05 12:23:07 assar Exp $");
 
 #define princ_num_comp(P) ((P)->name.name_string.len)
 #define princ_type(P) ((P)->name.name_type)
@@ -252,7 +254,7 @@ unparse_name(krb5_context context,
 	len++;
     }
     *name = malloc(len);
-    if(*name == NULL)
+    if(len != 0 && *name == NULL)
 	return ENOMEM;
     ret = unparse_name_fixed(context, principal, *name, len, short_flag);
     if(ret)
@@ -284,8 +286,7 @@ krb5_unparse_name_ext(krb5_context context,
 		      char **name,
 		      size_t *size)
 {
-    fprintf(stderr, "krb5_unparse_name_ext: not implemented\n");
-    abort();
+    krb5_abortx(context, "unimplemented krb5_unparse_name_ext called");
 }
 
 #endif
@@ -502,6 +503,54 @@ krb5_realm_compare(krb5_context context,
     return strcmp(princ_realm(princ1), princ_realm(princ2)) == 0;
 }
 
+struct v4_name_convert {
+    const char *from;
+    const char *to; 
+} default_v4_name_convert[] = {
+    { "ftp", "ftp" },
+    { "hprop", "hprop" },
+    { "pop", "pop" },
+    { "rcmd", "host" },
+    { NULL, NULL }
+};
+
+static const char*
+get_name_conversion(krb5_context context, const char *realm, const char *name)
+{
+    struct v4_name_convert *q;
+    const char *p;
+    p = krb5_config_get_string(context, NULL, "realms", realm,
+			       "v4_name_convert", "host", name, NULL);
+    if(p == NULL)
+	p = krb5_config_get_string(context, NULL, "libdefaults", 
+				   "v4_name_convert", "host", name, NULL);
+    if(p)
+	return p;
+
+    /* XXX should be possible to override default list */
+    p = krb5_config_get_string(context, NULL,
+			       "realms",
+			       realm,
+			       "v4_name_convert",
+			       "plain",
+			       name,
+			       NULL);
+    if(p)
+	return NULL;
+    p = krb5_config_get_string(context, NULL,
+			       "libdefaults",
+			       "v4_name_convert",
+			       "plain",
+			       name,
+			       NULL);
+    if(p)
+	return NULL;
+    for(q = default_v4_name_convert; q->from; q++)
+	if(strcmp(q->from, name) == 0)
+	    return q->to;
+    return NULL;
+}
+
 krb5_error_code
 krb5_425_conv_principal_ext(krb5_context context,
 			    const char *name,
@@ -530,11 +579,7 @@ krb5_425_conv_principal_ext(krb5_context context,
 	instance = NULL;
 	goto no_host;
     }
-    p = krb5_config_get_string(context, NULL, "realms", realm,
-			       "v4_name_convert", "host", name, NULL);
-    if(p == NULL)
-	p = krb5_config_get_string(context, NULL, "libdefaults", 
-				   "v4_name_convert", "host", name, NULL);
+    p = get_name_conversion(context, realm, name);
     if(p == NULL)
 	goto no_host;
     name = p;
@@ -660,9 +705,24 @@ krb5_425_conv_principal(krb5_context context,
 }
 
 
-static char*
+static int
+check_list(const krb5_config_binding *l, const char *name, const char **out)
+{
+    while(l){
+	if (l->type != krb5_config_string)
+	    continue;
+	if(strcmp(name, l->u.string) == 0) {
+	    *out = l->name;
+	    return 1;
+	}
+	l = l->next;
+    }
+    return 0;
+}
+
+static int
 name_convert(krb5_context context, const char *name, const char *realm, 
-	     const char *section)
+	     const char **out)
 {
     const krb5_config_binding *l;
     l = krb5_config_get_list (context,
@@ -670,24 +730,47 @@ name_convert(krb5_context context, const char *name, const char *realm,
 			      "realms",
 			      realm,
 			      "v4_name_convert",
-			      section,
+			      "host",
 			      NULL);
-    if(l == NULL)
-	l = krb5_config_get_list (context,
-				  NULL,
-				  "libdefaults",
-				  "v4_name_convert",
-				  section,
-				  NULL);
+    if(l && check_list(l, name, out))
+	return KRB5_NT_SRV_HST;
+    l = krb5_config_get_list (context,
+			      NULL,
+			      "libdefaults",
+			      "v4_name_convert",
+			      "host",
+			      NULL);
+    if(l && check_list(l, name, out))
+	return KRB5_NT_SRV_HST;
+    l = krb5_config_get_list (context,
+			      NULL,
+			      "realms",
+			      realm,
+			      "v4_name_convert",
+			      "plain",
+			      NULL);
+    if(l && check_list(l, name, out))
+	return KRB5_NT_UNKNOWN;
+    l = krb5_config_get_list (context,
+			      NULL,
+			      "libdefaults",
+			      "v4_name_convert",
+			      "host",
+			      NULL);
+    if(l && check_list(l, name, out))
+	return KRB5_NT_UNKNOWN;
     
-    while(l){
-	if (l->type != krb5_config_string)
-	    continue;
-	if(strcmp(name, l->u.string) == 0)
-	    return l->name;
-	l = l->next;
+    /* didn't find it in config file, try built-in list */
+    {
+	struct v4_name_convert *q;
+	for(q = default_v4_name_convert; q->from; q++) {
+	    if(strcmp(name, q->to) == 0) {
+		*out = q->from;
+		return KRB5_NT_SRV_HST;
+	    }
+	}
     }
-    return NULL;
+    return -1;
 }
 
 krb5_error_code
@@ -697,7 +780,7 @@ krb5_524_conv_principal(krb5_context context,
 			char *instance,
 			char *realm)
 {
-    char *n, *i, *r;
+    const char *n, *i, *r;
     char tmpinst[40];
     int type = princ_type(principal);
 
@@ -717,16 +800,11 @@ krb5_524_conv_principal(krb5_context context,
     }
 
     {
-	char *tmp = name_convert(context, n, r, "host");
-	if(tmp){
-	    type = KRB5_NT_SRV_HST;
+	const char *tmp;
+	int t = name_convert(context, n, r, &tmp);
+	if(t >= 0) {
+	    type = t;
 	    n = tmp;
-	}else{
-	    tmp = name_convert(context, n, r, "plain");
-	    if(tmp){
-		type = KRB5_NT_UNKNOWN;
-		n = tmp;
-	    }
 	}
     }
 
@@ -763,7 +841,8 @@ krb5_sname_to_principal (krb5_context context,
     krb5_error_code ret;
     char localhost[128];
     char **realms, *host = NULL;
-    
+    struct hostent *hp = NULL;
+	
     if(type != KRB5_NT_SRV_HST && type != KRB5_NT_UNKNOWN)
 	return KRB5_SNAME_UNSUPP_NAMETYPE;
     if(hostname == NULL){
@@ -772,27 +851,40 @@ krb5_sname_to_principal (krb5_context context,
     }
     if(sname == NULL)
 	sname = "host";
-    if(type == KRB5_NT_SRV_HST){
-	struct hostent *hp;
-	hp = roken_gethostbyname(hostname);
+    if(type == KRB5_NT_SRV_HST) {
+	int error;
+
+#ifdef HAVE_IPV6
+	if (hp == NULL)
+	    hp = getipnodebyname (hostname, AF_INET6, 0, &error);
+#endif
+	if (hp == NULL)
+	    hp = getipnodebyname (hostname, AF_INET, 0, &error);
 	if(hp != NULL)
 	    hostname = hp->h_name;
     }
-    if(type == KRB5_NT_SRV_HST){
+    if(type == KRB5_NT_SRV_HST) {
 	host = strdup(hostname);
 	if(host == NULL){
+	    if (hp != NULL)
+		freehostent (hp);
 	    return ENOMEM;
 	}
 	strlwr(host);
 	hostname = host;
     }
     ret = krb5_get_host_realm(context, hostname, &realms);
-    if(ret)
+    if(ret) {
+	if (hp != NULL)
+	    freehostent (hp);
 	return ret;
+    }
     ret = krb5_make_principal(context, ret_princ, realms[0], sname,
 			      hostname, NULL);
     if(host)
 	free(host);
+    if (hp)
+	freehostent (hp);
     krb5_free_host_realm(context, realms);
     return ret;
 }
