@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,31 +38,33 @@
 
 #include <krb5_locl.h>
 
-RCSID("$Id: rd_req.c,v 1.18 1997/07/16 21:24:55 joda Exp $");
+RCSID("$Id: rd_req.c,v 1.37 1999/04/20 14:18:11 assar Exp $");
 
 static krb5_error_code
 decrypt_tkt_enc_part (krb5_context context,
-		      const krb5_keyblock *key,
+		      krb5_keyblock *key,
 		      EncryptedData *enc_part,
 		      EncTicketPart *decr_part)
 {
     krb5_error_code ret;
     krb5_data plain;
     size_t len;
+    krb5_crypto crypto;
 
-    ret = krb5_decrypt (context,
-			enc_part->cipher.data,
-			enc_part->cipher.length,
-			enc_part->etype,
-			key, &plain);
+    krb5_crypto_init(context, key, 0, &crypto);
+    ret = krb5_decrypt_EncryptedData (context,
+				      crypto,
+				      KRB5_KU_TICKET,
+				      enc_part,
+				      &plain);
+    krb5_crypto_destroy(context, crypto);
     if (ret)
 	return ret;
 
-    ret = decode_EncTicketPart(plain.data, plain.length, decr_part, &len);
+    ret = krb5_decode_EncTicketPart(context, plain.data, plain.length, 
+				    decr_part, &len);
     krb5_data_free (&plain);
-    if (ret)
-	return ret;
-    return 0;
+    return ret;
 }
 
 static krb5_error_code
@@ -74,20 +76,22 @@ decrypt_authenticator (krb5_context context,
     krb5_error_code ret;
     krb5_data plain;
     size_t len;
+    krb5_crypto crypto;
 
-    ret = krb5_decrypt (context,
-			enc_part->cipher.data,
-			enc_part->cipher.length,
-			enc_part->etype,
-			key, &plain);
+    krb5_crypto_init(context, key, 0, &crypto);
+    ret = krb5_decrypt_EncryptedData (context,
+				      crypto,
+				      KRB5_KU_AP_REQ_AUTH,
+				      enc_part,
+				      &plain);
+    krb5_crypto_destroy(context, crypto);
     if (ret)
 	return ret;
 
-    ret = decode_Authenticator(plain.data, plain.length, authenticator, &len);
+    ret = krb5_decode_Authenticator(context, plain.data, plain.length, 
+				    authenticator, &len);
     krb5_data_free (&plain);
-    if (ret) 
-	return ret;
-    return 0;
+    return ret;
 }
 
 krb5_error_code
@@ -112,17 +116,90 @@ krb5_decode_ap_req(krb5_context context,
 	free_AP_REQ(ap_req);
 	return KRB5KRB_AP_ERR_BADVERSION;
     }
-    if (ap_req->ap_options.use_session_key)
-	abort ();
     return 0;
 }
 
+krb5_error_code
+krb5_decrypt_ticket(krb5_context context,
+		    Ticket *ticket,
+		    krb5_keyblock *key,
+		    EncTicketPart *out,
+		    krb5_flags flags)
+{
+    EncTicketPart t;
+    krb5_error_code ret;
+    ret = decrypt_tkt_enc_part (context, key, &ticket->enc_part, &t);
+    if (ret)
+	return ret;
+    
+    {
+	int32_t now;
+	time_t start = t.authtime;
+
+	krb5_timeofday (context, &now);
+	if(t.starttime)
+	    start = *t.starttime;
+	if(start - now > context->max_skew
+	   || (t.flags.invalid
+	       && !(flags & KRB5_VERIFY_AP_REQ_IGNORE_INVALID)))
+	    return KRB5KRB_AP_ERR_TKT_NYV;
+	if(now - t.endtime > context->max_skew)
+	    return KRB5KRB_AP_ERR_TKT_EXPIRED;
+    }
+    
+    if(out)
+	*out = t;
+    else
+	free_EncTicketPart(&t);
+    return 0;
+}
+
+krb5_error_code
+krb5_verify_authenticator_checksum(krb5_context context,
+				   krb5_auth_context ac,
+				   void *data,
+				   size_t len)
+{
+    krb5_error_code ret;
+    krb5_keyblock *key;
+    krb5_authenticator authenticator;
+    krb5_crypto crypto;
+    
+    ret = krb5_auth_getauthenticator (context,
+				      ac,
+				      &authenticator);
+    if(ret)
+	return ret;
+    if(authenticator->cksum == NULL)
+	return -17;
+    ret = krb5_auth_con_getkey(context, ac, &key);
+    if(ret) {
+	krb5_free_authenticator(context, &authenticator);
+	return ret;
+    }
+    ret = krb5_crypto_init(context, key, 0, &crypto);
+    if(ret)
+	goto out;
+    ret = krb5_verify_checksum (context,
+				crypto,
+				KRB5_KU_AP_REQ_AUTH_CKSUM,
+				data,
+				len,
+				authenticator->cksum);
+    krb5_crypto_destroy(context, crypto);
+out:
+    krb5_free_authenticator(context, &authenticator);
+    krb5_free_keyblock(context, key);
+    return ret;
+}
+
 krb5_error_code
 krb5_verify_ap_req(krb5_context context,
 		   krb5_auth_context *auth_context,
 		   krb5_ap_req *ap_req,
 		   krb5_const_principal server,
 		   krb5_keyblock *keyblock,
+		   krb5_flags flags,
 		   krb5_flags *ap_req_options,
 		   krb5_ticket **ticket)
 {
@@ -139,23 +216,30 @@ krb5_verify_ap_req(krb5_context context,
     }else
 	krb5_auth_con_init(context, &ac);
 	
-    ret = decrypt_tkt_enc_part (context,
-				keyblock,
-				&ap_req->ticket.enc_part,
-				&t.ticket);
-    if (ret)
-	return ret;
+    if (ap_req->ap_options.use_session_key && ac->keyblock){
+	ret = krb5_decrypt_ticket(context, &ap_req->ticket, 
+				  ac->keyblock, 
+				  &t.ticket,
+				  flags);
+	krb5_free_keyblock(context, ac->keyblock);
+	ac->keyblock = NULL;
+    }else
+	ret = krb5_decrypt_ticket(context, &ap_req->ticket, 
+				  keyblock, 
+				  &t.ticket,
+				  flags);
     
-    principalname2krb5_principal(&t.server,
-				 ap_req->ticket.sname,
-				 ap_req->ticket.realm);
+    if(ret)
+	return ret;
 
-    principalname2krb5_principal(&t.client,
-				 t.ticket.cname,
+    principalname2krb5_principal(&t.server, ap_req->ticket.sname, 
+				 ap_req->ticket.realm);
+    principalname2krb5_principal(&t.client, t.ticket.cname, 
 				 t.ticket.crealm);
+
     /* save key */
 
-    copy_EncryptionKey(&t.ticket.key, &ac->key);
+    krb5_copy_keyblock(context, &t.ticket.key, &ac->keyblock);
 
     ret = decrypt_authenticator (context,
 				 &t.ticket.key,
@@ -201,8 +285,9 @@ krb5_verify_ap_req(krb5_context context,
     /* And where should it be stored? */
 
     if (ac->authenticator->subkey) {
-	copy_EncryptionKey (ac->authenticator->subkey,
-			    &ac->remote_subkey);
+	krb5_copy_keyblock(context, 
+			   ac->authenticator->subkey,
+			   &ac->remote_subkey);
     }
 
     if (ap_req_options) {
@@ -213,22 +298,11 @@ krb5_verify_ap_req(krb5_context context,
 	    *ap_req_options |= AP_OPTS_MUTUAL_REQUIRED;
     }
 
-    {
-	time_t now = time (NULL);
-	time_t start = t.ticket.authtime;
-	if(t.ticket.starttime)
-	    start = *t.ticket.starttime;
-	if(start - now > context->max_skew || t.ticket.flags.invalid)
-	    return KRB5KRB_AP_ERR_TKT_NYV;
-	if(now - t.ticket.endtime > context->max_skew)
-	    return KRB5KRB_AP_ERR_TKT_EXPIRED;
-    }
-    
     if(ticket){
 	*ticket = malloc(sizeof(**ticket));
 	**ticket = t;
     } else
-	free_EncTicketPart(&t.ticket);
+	krb5_free_ticket (context, &t);
     return 0;
 }
 		   
@@ -244,8 +318,6 @@ krb5_rd_req_with_keyblock(krb5_context context,
 {
     krb5_error_code ret;
     krb5_ap_req ap_req;
-    size_t len;
-    struct timeval now;
 
     if (*auth_context == NULL) {
 	ret = krb5_auth_con_init(context, auth_context);
@@ -256,18 +328,60 @@ krb5_rd_req_with_keyblock(krb5_context context,
     ret = krb5_decode_ap_req(context, inbuf, &ap_req);
     if(ret)
 	return ret;
-    
+
     ret = krb5_verify_ap_req(context,
 			     auth_context,
 			     &ap_req,
 			     server,
 			     keyblock,
+			     0,
 			     ap_req_options,
 			     ticket);
+
     free_AP_REQ(&ap_req);
     return ret;
 }
 
+static krb5_error_code
+get_key_from_keytab(krb5_context context,
+		    krb5_auth_context *auth_context,
+		    krb5_ap_req *ap_req,
+		    krb5_const_principal server,
+		    krb5_keytab keytab,
+		    krb5_keyblock **out)
+{
+    krb5_keytab_entry entry;
+    krb5_error_code ret;
+    int kvno;
+    krb5_keytab real_keytab;
+
+    if(keytab == NULL)
+	krb5_kt_default(context, &real_keytab);
+    else
+	real_keytab = keytab;
+    
+    if (ap_req->ticket.enc_part.kvno)
+	kvno = *ap_req->ticket.enc_part.kvno;
+    else
+	kvno = 0;
+
+    ret = krb5_kt_get_entry (context,
+			     real_keytab,
+			     server,
+			     kvno,
+			     ap_req->ticket.enc_part.etype,
+			     &entry);
+    if(ret)
+	goto out;
+    ret = krb5_copy_keyblock(context, &entry.keyblock, out);
+    krb5_kt_free_entry (context, &entry);
+out:    
+    if(keytab == NULL)
+	krb5_kt_close(context, real_keytab);
+    
+    return ret;
+}
+
 krb5_error_code
 krb5_rd_req(krb5_context context,
 	    krb5_auth_context *auth_context,
@@ -277,24 +391,56 @@ krb5_rd_req(krb5_context context,
 	    krb5_flags *ap_req_options,
 	    krb5_ticket **ticket)
 {
-    krb5_keytab_entry entry;
     krb5_error_code ret;
-    if(keytab == NULL)
-	krb5_kt_default(context, &keytab);
-    ret = krb5_kt_get_entry(context,
-			    keytab,
-			    (krb5_principal)server,
-			    0,
-			    KEYTYPE_DES,
-			    &entry);
+    krb5_ap_req ap_req;
+    krb5_keyblock *keyblock = NULL;
+    krb5_principal service = NULL;
+
+    if (*auth_context == NULL) {
+	ret = krb5_auth_con_init(context, auth_context);
+	if (ret)
+	    return ret;
+    }
+
+    ret = krb5_decode_ap_req(context, inbuf, &ap_req);
     if(ret)
 	return ret;
-    
-    return krb5_rd_req_with_keyblock(context,
-				     auth_context,
-				     inbuf,
-				     server,
-				     &entry.keyblock,
-				     ap_req_options,
-				     ticket);
+
+    if(server == NULL){
+	principalname2krb5_principal(&service,
+				     ap_req.ticket.sname,
+				     ap_req.ticket.realm);
+	server = service;
+    }
+
+    if(ap_req.ap_options.use_session_key == 0 || 
+       (*auth_context)->keyblock == NULL){
+	ret = get_key_from_keytab(context, 
+				  auth_context, 
+				  &ap_req,
+				  server,
+				  keytab,
+				  &keyblock);
+	if(ret)
+	    goto out;
+    }
+	
+
+    ret = krb5_verify_ap_req(context,
+			     auth_context,
+			     &ap_req,
+			     server,
+			     keyblock,
+			     0,
+			     ap_req_options,
+			     ticket);
+
+    if(keyblock != NULL)
+	krb5_free_keyblock(context, keyblock);
+
+out:
+    free_AP_REQ(&ap_req);
+    if(service)
+	krb5_free_principal(context, service);
+    return ret;
 }
